--[[

̃t@CRampageAÏꕔłB

RampageAI̓t[\tgł; Free Software FoundationzGNUʌOp_񏑂̏̂ƂōĔzzщς邱Ƃł܂;
GNUCZXo[W2A܂(Ȃ̑I)2ȑÕo[WɂāB

RampageAÍALpłƂ]Ĕzz܂B
*ۏ؂Ȃ*; s̐A̖ړÎ߂̓K̈ÖٓIȕۏ؂ȂĂB 
̑̏ڍׂɊւẮAGNUʌOp_񏑂ĂB

RampageAIƋGNUʌOp_񏑂̃Rs[𓯎Ɏ󂯎ׂł;
łȂȂAȉ̏֏ȂB
Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

--]]

-------------------------------
-- Monster Selection Support --
-------------------------------

-- Check if we're allowed to target the monster
function MonsterAllowed(actor)
	-- Check if either skills or melee are allowed against the monster 
	if MSkillsAllowed(actor) or MAttackAllowed(actor) then
		return true
	end
	
	return false
end

-- Convert the MonsterDB table
function ConvertMDB()
	-- Log it
	Log("Optimizing monster database...")
	
	-- Verify we have a valid default
	if type(DefaultMonsterSettings) == "table" then
		-- Remove the "ID" from our default settings
		Table.remove(DefaultMonsterSettings,1)
	else
		-- Generate a blank table
		DefaultMonsterSettings = { }
	end
	
	-- Verify our default settings
	DefaultMonsterSettings = VerifyMonSettings(DefaultMonsterSettings,true)

	-- Make a copy of the monster db
	local mdb = { }

	-- Loop through each monster type
	local i,t
	for i,t in pairs(MonsterDB) do
		-- Grab the ID
		local id = t[1]
		
		-- Verify the ID
		if type(id) == "number" then
			-- Remove the first entry from the table (ID)
			Table.remove(t,1)
			
			-- Add it to our new database
			mdb[id] = VerifyMonSettings(t)
		end
	end
	
	MonsterDB = mdb
	
	-- Ensure this isn't repeated
	ConvertMDB = NoReturn
end

-- Verify monster settings
function VerifyMonSettings(t,isDefault)
	-- Check if we're working on the default
	local def
	if isDefault then
		def = { 1, true, true, 2000 }
	else
		def = DefaultMonsterSettings
	end
	
	-- Make the table of types
	local types = { }
	local i,val
	for i,val in pairs(def) do
		-- Get the type of value, and set it in our types
		types[i] = type(val)
		
		-- Check if theres a value in our table
		if t[i] == nil then
			-- Set the option
			t[i] = val
		end
	end
	
	-- Loop through each value of the monster table
	local i,val
	for i,val in pairs(t) do
		-- Check if theres an option in our types
		if types[i] == nil then
			-- No type, so remove the option
			t[i] = nil
		else
			-- Theres an option, check its type
			if type(val) ~= types[i] then
				-- Wrong type, so set it to our default
				t[i] = def[i]
			end
		end
	end
	
	-- Return our verified table
	return t
end

-- Get the monster-specific table
function MDBTable(actor)
	-- Get the actor type
	local type = GetV(V_HOMUNTYPE,actor)
	
	-- Check if this is monster-specific
	if MonsterDB[type] ~= nil then
		-- It is, return the monster-specific table
		return MonsterDB[type]
	end
	
	-- Its not monster-specific, return the default
	return DefaultMonsterSettings
end

-- Check if we are allowed to use skills against the monster
function MSkillsAllowed(actor)
	-- Get the monster's type
	local t = MDBTable(actor)
	
	-- Return the skills-allowed value
	return t[3]
end

-- Check the monster-specific skill timeout
function MSkillTimeout(actor)
	-- Get the monster's type
	local t = MDBTable(actor)
	
	-- Return the timeout value
	return t[4]
end

-- Check if we are allowed to melee the monster
function MAttackAllowed(actor)
	-- Get the monster's type
	local t = MDBTable(actor)
	
	-- Return the melee allowed value
	return t[2]
end

-- Check the priority level of a monster
function MAttackPriority(actor)
	-- Check the mosnter's type
	local t = MDBTable(actor)
	
	-- Return the priority level
	return t[1]
end

-- Function to select the highest priority monsters from a list
function MPrioritySieve(t)
	-- Get the highest value of the table
	local max = Table.getn(t)
	
	-- Check if we have none
	if max == 0 then
		return Table.new()
	end

	-- Setup the default table
	local priority = -100			-- Hopefully none have lower than -100
	local monsters = Table.new()
	
	-- Loop through our table, sorting the highest priority monsters into our new table
	local i,actor,prio
	for i=1,max do
		-- Get the actor
		actor = t[i]
		
		-- Check the actor's priority
		prio = MAttackPriority(actor)
		
		-- Check if we should start a new priority table
		if prio > priority then
			-- Priority is higher, start a new table
			priority = prio
			monsters = Table.new()
		end
		
		-- Check if we should add the monster to the table
		if prio == priority then
			-- Add the monster
			Table.insert(monsters,actor)
		end
	end
	
	-- Return the table with the highest priority monsters
	return monsters
end

----------------------------
-- Monster Ignore Support --
----------------------------

-- Remove all the ignored monsters from a Table
function IgnoredSieve(t)
	-- Loop through each monster in the table
	local i,max = 1,Table.getn(t)
	while i < max do
		-- Check if the monster is ignored
		if IsIgnored(t[i]) then
			-- Monster is ignored, remove it
			Table.remove(t,i)
		else
			-- Monster isn't ignored, move to next
			i = i + 1
		end
	end

	return t
end

-- Check our list of ignored monsters
function CheckIgnored(ticks)
	-- Loop through all of our ignored monsters
	local i,max = 1,Table.getn(IgnoredTargets)
	if max < 1 then
		return
	end
	repeat
		-- Subtract the tick-change from our remaining ignore time
		IgnoredTargets[i][1] = IgnoredTargets[i][1] - ticks
		
		-- Log it
		--Log("Removed %d ticks from Actor #%d's ignore time, resulting in %d",
		--	ticks, IgnoredTargets[i][2], IgnoredTargets[i][1])

		-- Check if the target should be ignored any longer
		if IgnoredTargets[i][1] <= 0 then
			-- Remove it
			t = Table.remove(IgnoredTargets,i)
			max = Table.getn(IgnoredTargets)
		else
			-- Next!
			i = i + 1
		end
	until i >= max
end

-- Check if a monster is ignored
function IsIgnored(actor)
	-- Loop through our ignored monsters
	local i
	for i=1,Table.getn(IgnoredTargets) do
		-- Check if the monster ID is in the ignored list
		if IgnoredTargets[i][2] == actor then
			return true
		end
	end

	-- If its not in the ignored list, its not ignored
	return false
end

-- Add a monster to the ignore list
function AddIgnore(actor)
	-- Check if its already ignored
	if IsIgnored(actor) then
		-- Do nothing
		return
	end
	
	-- Log it
	Log("Ignoring actor #%d for %d milliseconds.",actor,ActorIgnoreTime)

	-- Add the actor to the ignore list
	Table.insert(IgnoredTargets,{ ActorIgnoreTime, actor })
end